import json
import logging

from pysamsungrac.network import SamsungRacConnection as Conn
from pysamsungrac.exceptions import DeviceMissingToken
_LOGGER = logging.getLogger(__name__)


class SamsungRac:
    """Class representing a physical device, it's state and properties.

        Devices must have a token to authenticate with

        Once a device is bound occasionally call `update` to request and update state from
        the HVAC, as it is possible that it changes state from other sources.

        Attributes:
            name: Current name of the HVAC device
            description: Usually the model number of the external unit
            uuid: The unique identifier on the samsung cloud
            power: A string indicating if the unit is on or off
            temperature unit: An string indicating temperature units
            target_temperature: The target temperature
            current_temperature: The current temperature as reported by the indoor unit
            max_temp: An int indicating maximum supported temperature
            min_temp: An int indicating maximum supported temperature
            hvac_mode: The current selected operating mode
            hvac_modes: Supported hvac modes
            fan_speed: int indicating current fan speed
            fan_speeds: int indicating supported fan speeds
            swing_mode: current swing mode
            swing_modes: supported swing modes
            preset_mode: current preset mode
            preset_modes: supported preset modes
            sleep_timer: int for setting sleep time for the Sleep preset. Setting this automatically engages Sleep preset
            purifier: Indicates of the ionizer is on or off
            filter_time: Indicates the time since last filter alarm reset
            filter_alarm_time: Number of hours for the filter alarm to trigger
            filter_alarm: Status of the filter alarm
            autoclean: Start autoclean on the indoor unit
            cool_capability: Cooling power expressed in kW as reported by the outdoor unit
            warm_capability: Heating power expressed in kW as reported by the outdoor unit
            options: a dict of untranslated options of the device
        """

    ENDPOINTS = {
        'Operation': '/devices/0',
        'Wind': '/devices/0/wind',
        'Temperature': '/devices/0/temperatures/0',
        'Modes': '/devices/0/mode',
        'Energy': '/files/usage.db'
    }

    SWING_MODES = [
        'Fix',
        'Up_And_Low',
        'Left_And_Right',
        'All'
    ]

    COMODE_MODES = [
        'Off',
        'Sleep',  # Good Sleep
        '2Step',
        'Speed',  # Fast Turbo
        'Comfort',
        'Quiet',
        'Smart'  # Single User
    ]

    def __init__(self, address, token=None):
        self._address = address
        self._token = token
        self._control = Conn(address, token)
        self._name = None
        self._description = None
        self._uuid = None
        self._state = {}

    async def _set(self, endpoint, command):
        await self._control.put(
            command=command,
            resource=self.ENDPOINTS[endpoint],
        )

    async def update(self):
        if not self._token:
            raise DeviceMissingToken

        _LOGGER.debug("Updating device infor for (%s)", str(self._address))

        ac_data = json.loads(await self._control.get(resource=self.ENDPOINTS['Operation']))['Device']

        self._name = ac_data['name']
        self._description = ac_data['description']
        self._uuid = ac_data['uuid']
        self._state['power'] = ac_data['Operation']['power']
        self._state['temperature_unit'] = ac_data['Temperatures'][0]['unit']
        self._state['current_temperature'] = ac_data['Temperatures'][0]['current']
        self._state['target_temperature'] = ac_data['Temperatures'][0]['desired']
        self._state['max_temp'] = ac_data['Temperatures'][0]['maximum']
        self._state['min_temp'] = ac_data['Temperatures'][0]['minimum']
        self._state['hvac_mode'] = str(ac_data['Mode']['modes']).strip("'[]")
        self._state['hvac_modes'] = ac_data['Mode']['supportedModes']
        self._state['fan_speed'] = ac_data['Wind']['speedLevel']
        self._state['fan_speeds'] = ac_data['Wind']['maxSpeedLevel']
        self._state['swing_mode'] = ac_data['Wind']['direction']
        self._state['options'] = {x.split('_')[0]: x.split('_')[1] for x in ac_data['Mode']['options']}

    def get_attribute(self, attribute):
        return self._state[attribute]

    def get_option(self, option):
        return self._state['options'][option]

    async def set_power(self, state: str):
        data = {'Operation': {'power': state}}
        return await self._set("Operation", data)

    async def set_hvac_mode(self, mode: str):
        data = {'modes': [mode]}
        return await self._set('Modes', data)

    async def set_target_temperature(self, temperature: int):
        data = {'desired': int(temperature)}
        return await self._set('Temperature', data)

    async def set_fan_speed(self, speed: int):
        data = {'speedLevel': int(speed)}
        return await self._set('Wind', data)

    async def set_swing_mode(self, mode: str):
        data = {'direction': mode}
        return await self._set('Wind', data)

    async def set_option(self, option: str, value: str):
        data = {'options': [f'{option}_{value}']}
        return await self._set('Modes', data)

    async def set_preset(self, value: str):
        return await self.set_option('Comode', value)

    async def set_sleep_timer(self, value: int):
        return await self.set_option('Sleep', str(value))

    async def set_purifier(self, value: str):
        return await self.set_option('Spi', value)

    async def set_beep(self, value:str):
        if value == 'On':
            return await self.set_option('Volume', '100')
        else:
            return await self.set_option('Volume', 'Mute')

    async def set_filter_alarm_time(self, value):
        return await self.set_option('FilterAlarmTime', value)

    async def reset_filter(self):
        # Not implemented yet
        pass

    async def set_clean(self, value):
        return await self.set_option('Autoclean', value)

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def uuid(self) -> str:
        return self._uuid

    @property
    def power(self) -> str:
        return self.get_attribute('power')

    @property
    def temperature_unit(self) -> str:
        return self.get_attribute('temperature_unit')

    @property
    def current_temperature(self) -> int:
        return int(self.get_attribute('current_temperature'))

    @property
    def target_temperature(self) -> int:
        return int(self.get_attribute('target_temperature'))

    @property
    def max_temp(self) -> int:
        return int(self.get_attribute('max_temp'))

    @property
    def min_temp(self) -> int:
        return int(self.get_attribute('min_temp'))

    @property
    def hvac_mode(self) -> str:
        return self.get_attribute('hvac_mode')

    @property
    def hvac_modes(self) -> list:
        modes = self.get_attribute('hvac_modes')
        # The API lies
        modes.append('Heat')
        return modes

    @property
    def fan_speed(self) -> int:
        return int(self.get_attribute('fan_speed'))

    @property
    def fan_speeds(self) -> list:
        return list(range(self.get_attribute('fan_speeds') + 1))

    @property
    def swing_mode(self) -> str:
        return self.get_attribute('swing_mode')

    @property
    def swing_modes(self) -> list:
        return self.SWING_MODES

    @property
    def preset_mode(self) -> str:
        return self.get_option('Comode')

    @property
    def preset_modes(self) -> list:
        return self.COMODE_MODES

    @property
    def sleep_timer(self) -> int:
        return int(self.get_option('Sleep'))

    @property
    def purifier(self) -> str:
        return self.get_option('Spi')

    @property
    def filter_time(self) -> float:
        return float(self.get_option('FilterTime')) / 10

    @property
    def filter_alarm_time(self) -> int:
        return int(self.get_option('FilterAlarmTime'))

    @property
    def filter_alarm(self) -> int:
        return self.get_option('FilterCleanAlarm')

    @property
    def autoclean(self) -> str:
        return self.get_option('Autoclean')

    @property
    def beep(self) -> str:
        if self.get_option('Volume') == 'Mute':
            return "Off"
        else:
            return "On"

    @property
    def cool_capability(self) -> float:
        return float(self.get_option('CoolCapa')) / 10

    @property
    def warm_capability(self) -> float:
        return float(self.get_option('WarmCapa')) / 10

    @property
    def options(self) -> dict:
        return self._state['options']



#!/usr/bin/env python3
import asyncio
import argparse

from pysamsungrac.samsungrac import SamsungRac


def get_attr(args):
    if args.name or args.all:
        print("Name: " + rac.name)
    if args.description or args.all:
        print("Description: " + rac.description)
    if args.power or args.all:
        print("Power: " + rac.power)
    if args.current_temperature or args.all:
        print("Current temperature: " + str(rac.current_temperature) + " " + rac.temperature_unit)
    if args.target_temperature or args.all:
        print("Target temperature: " + str(rac.target_temperature) + " " + rac.temperature_unit)
    if args.minimum_temperature or args.all:
        print("Minimum temperature: " + str(rac.min_temp) + " " + rac.temperature_unit)
    if args.maximum_temperature or args.all:
        print("Maximum temperature: " + str(rac.max_temp) + " " + rac.temperature_unit)
    if args.hvac_mode or args.all:
        print("HVAC mode: " + rac.hvac_mode)
    if args.hvac_modes or args.all:
        print("HVAC modes: " + str(rac.hvac_modes))
    if args.fan_speed or args.all:
        print("Fan speed: " + str(rac.fan_speed))
    if args.fan_speeds or args.all:
        print("Fan speed: " + str(rac.fan_speeds))
    if args.swing_mode or args.all:
        print("Swing mode: " + rac.swing_mode)
    if args.swing_modes or args.all:
        print("Swing modes: " + str(rac.swing_modes))
    if args.preset_mode or args.all:
        print("Preset mode: " + rac.preset_mode)
    if args.preset_modes or args.all:
        print("Preset modes: " + str(rac.preset_modes))
    if args.sleep_timer or args.all:
        # TODO: 0 - 5 is multiples of 30 mins, so 1 is 30 mins, 2 is 1h, 3 is 1h 30m, up to 2h 30m
        # 6 is 3 h, up to 24 which is 12h
        print("Sleep timer: " + str(rac.sleep_timer))
    if args.purifier or args.all:
        print("Purifier: " + str(rac.purifier))
    if args.filter_time or args.all:
        print("Filter time: " + str(rac.filter_time))
    if args.filter_alarm_time or args.all:
        print("Filter alarm time: " + str(rac.filter_alarm_time))
    if args.filter_alarm or args.all:
        print("Filter alarm: " + str(rac.filter_alarm))
    if args.beep or args.all:
        print("Beep: " + rac.beep)
    if args.cool_capability or args.all:
        print("Cool Capability: " + str(rac.cool_capability) + "kW")
    if args.warm_capability or args.all:
        print("Warm Capability: " + str(rac.warm_capability) + "kW")
    if args.clean or args.all:
        print("Clean: " + rac.autoclean)
    if args.options or args.all:
        print("Options: " + str(rac.options))


def set_attr(args):
    if args.power:
        loop.run_until_complete(rac.set_power(args.power))
    if args.hvac_mode:
        if args.hvac_mode in rac.hvac_modes:
            loop.run_until_complete(rac.set_hvac_mode(args.hvac_mode))
        else:
            raise ValueError('Mode not supported')
    if args.target_temperature:
        if args.target_temperature in range(rac.min_temp, rac.max_temp + 1):
            loop.run_until_complete(rac.set_target_temperature(args.target_temperature))
        else:
            raise ValueError('Temperature out of range')
    if args.fan_speed:
        if args.fan_speed in rac.fan_speeds:
            loop.run_until_complete(rac.set_fan_speed(args.fan_speed))
        else:
            raise ValueError('Fan speed out of range')
    if args.swing_mode:
        if args.swing_mode in rac.swing_modes:
            loop.run_until_complete(rac.set_swing_mode(args.swing_mode))
        else:
            raise ValueError('Invalid swing mode')
    if args.preset_mode:
        if args.preset_mode in rac.preset_modes:
            loop.run_until_complete(rac.set_preset(args.preset_mode))
        else:
            raise ValueError('Invalid preset')
    if args.sleep_timer:
        if args.sleep_timer in range(1, 24 + 1):
            loop.run_until_complete(rac.set_sleep_timer(args.sleep_timer))
        else:
            raise ValueError('Timer out of range')
    if args.purifier:
        loop.run_until_complete(rac.set_purifier(args.purifier))
    if args.beep:
        loop.run_until_complete(rac.set_beep(args.beep))
    if args.filter_alarm_time:
        loop.run_until_complete(rac.set_filter_alarm_time(args.filter_alarm_time))
    if args.reset_filter:
        print("Sorry, not implemented yet")
    if args.clean:
        loop.run_until_complete(rac.set_clean(args.clean))


parser = argparse.ArgumentParser(description="Samsung RAC wireless interface")
parser.add_argument('--address', required=True)
parser.add_argument('--token', required=True)

# Main commands
subparsers = parser.add_subparsers(help='supported commands')
parser_get = subparsers.add_parser("get")
parser_set = subparsers.add_parser("set")
parser_get_token = subparsers.add_parser("get_token")

# Set commands
parser_set.set_defaults(func=set_attr)
parser_set.add_argument('--power', choices=['On','Off'], help='Set power state')
parser_set.add_argument('--hvac_mode', help='Set HVAC mode')
parser_set.add_argument('--target_temperature', help='Set target temperature', type=int)
parser_set.add_argument('--fan_speed', help='Set fan speed', type=int)
parser_set.add_argument('--swing_mode', help='Set swing mode')
parser_set.add_argument('--preset_mode', help='Set preset mode')
parser_set.add_argument('--sleep_timer', help='Set sleep timer', type=int)
parser_set.add_argument('--purifier', choices=['On','Off'], help='Enable air purifier')
parser_set.add_argument('--beep', choices=['On','Off'], help='Enable/disable beep')
parser_set.add_argument('--filter_alarm_time', help='Set filter alarm time', type=int)
parser_set.add_argument('--reset_filter', action='store_true', help='Reset filter alarm')
parser_set.add_argument('--clean', choices=['On','Off'], help='Set unit autoclean')

# Get commands
parser_get.set_defaults(func=get_attr)
parser_get.add_argument('--all', action='store_true', help='shows all supported attributes')
parser_get.add_argument('--name', action='store_true', help='Name of the AC unit')
parser_get.add_argument('--description', action='store_true', help='Description of the AC unit')
parser_get.add_argument('--power', action='store_true', help='Current power state')
parser_get.add_argument('--current_temperature', action='store_true', help='Current room temperature')
parser_get.add_argument('--target_temperature', action='store_true', help='Current target temperature')
parser_get.add_argument('--maximum_temperature', action='store_true', help='Maximum supported temperature')
parser_get.add_argument('--minimum_temperature', action='store_true', help='Minimum supported temperature')
parser_get.add_argument('--hvac_mode', action='store_true', help='Current HVAC mode')
parser_get.add_argument('--hvac_modes', action='store_true', help='Supported HVAC modes')
parser_get.add_argument('--fan_speed', action='store_true', help='Current fan speed')
parser_get.add_argument('--fan_speeds', action='store_true', help='Supported fan speeds')
parser_get.add_argument('--swing_mode', action='store_true', help='Current swing mode')
parser_get.add_argument('--swing_modes', action='store_true', help='Supported swing modes')
parser_get.add_argument('--preset_mode', action='store_true', help='Current preset mode')
parser_get.add_argument('--preset_modes', action='store_true', help='Supported preset modes')
parser_get.add_argument('--sleep_timer', action='store_true', help='Sleep timer value')
parser_get.add_argument('--cool_capability', action='store_true', help='Cooling capability of the unit')
parser_get.add_argument('--warm_capability', action='store_true', help='Heating capability of the unit')
parser_get.add_argument('--purifier', action='store_true', help='Current state of air purifier')
parser_get.add_argument('--beep', action='store_true', help='State of the beeper')
parser_get.add_argument('--filter_time', action='store_true', help='Amount of time since last filter cleaning')
parser_get.add_argument('--filter_alarm_time', action='store_true', help='Filter alarm time')
parser_get.add_argument('--filter_alarm', action='store_true', help='Filter cleaning alarm')
parser_get.add_argument('--clean', action='store_true', help='Unit self-cleaning')
parser_get.add_argument('--options', action='store_true', help='All unit suboptions')

args = parser.parse_args()
loop = asyncio.get_event_loop()
rac = SamsungRac(args.address, args.token)
loop.run_until_complete(rac.update())
args.func(args)



import asyncio
import aiohttp
import os
import ssl
import logging

from aiohttp import ClientSession

from pysamsungrac.exceptions import DeviceTimeoutError, DeviceCommuncationError

DEFAULT_TIMEOUT = 10
_LOGGER = logging.getLogger(__name__)


class SamsungRacToken:
    def __init__(self, address):
        pass

    # async def token_server(self):
    #     """Start webserver"""
    #     cert = os.path.join(os.path.dirname(__file__), 'ac14k_m.pem')
    #     ssl_ctx = ssl.create_default_context(ssl.Purpose.SERVER_AUTH, cafile=cert)
    #     ssl_ctx.load_cert_chain(cert)
    #     server = aiohttp.web.Application()

    # async def request_token(self):
    #     """Create server listener here"""
    #
    #     resource = '/devicetoken/request'
    #     try:
    #         await self._session.post(self._url + resource)
    #     except asyncio.TimeoutError:
    #         _LOGGER.error('Timed out requesting command to SamsungRac')
    #     except aiohttp.ClientResponseError as error:
    #         _LOGGER.error('Error requesting token from SamsungRac: %s', error.message)
    #     _LOGGER.info('Turn on the unit now!')


class SamsungRacConnection:
    """Class to communicate with the device"""

    def __init__(self, address, token, timeout=DEFAULT_TIMEOUT, session=None) -> None:
        self._loop = asyncio.get_event_loop()
        if session is None:
            self._session = self._loop.run_until_complete(self._create_session())
        else:
            self._session = session
        self._url = f'https://{address}:8888'
        self._timeout = timeout
        self._headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
        }

    def __del__(self):
        self._loop.run_until_complete(self._session.close())

    @staticmethod
    async def _create_session() -> ClientSession:
        cert = os.path.join(os.path.dirname(__file__), 'ac14k_m.pem')
        ssl_ctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH, cafile=cert)
        ssl_ctx.load_cert_chain(cert)
        return aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=ssl_ctx), raise_for_status=True)

    async def get(self, resource):
        _LOGGER.debug('GET: %s', self._url + resource)
        try:
            resp = await self._session.get(self._url + resource, headers=self._headers)
        except aiohttp.ServerTimeoutError as e:
            raise DeviceTimeoutError(e)
        except aiohttp.ClientResponseError as e:
            raise DeviceCommuncationError(e)
        return await resp.text()

    async def put(self, resource, command):
        _LOGGER.debug('PUT: %s, %s', self._url + resource, command)
        try:
            await self._session.put(self._url + resource, json=command, headers=self._headers)
        except aiohttp.ServerTimeoutError as e:
            raise DeviceTimeoutError(e)
        except aiohttp.ClientResponseError as e:
            raise DeviceCommuncationError(e)



class DeviceCommuncationError(Exception):
    """The token we're using is wrong"""

    pass


class DeviceTimeoutError(Exception):
    """The device timed out when trying to communicate"""

    pass

class DeviceMissingToken(Exception):
    """We don't have a token to communicate with"""